@@ -28,6 +28,7 @@
 #include <inttypes.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <sys/sysmacros.h>
 #include <sys/ioctl.h>
 #include <linux/hdreg.h>
 #include <errno.h>
@@ -132,6 +133,9 @@ read_dasd_pt(int fd, struct slice all, struct slice *sp, int ns)
 			/* Couldn't open the device */
 			return -1;
 		}
+	} else if ((unsigned int)major(sbuf.st_rdev) != 94) {
+			/* Not a DASD */
+			return -1;
 	} else {
 		fd_dasd = fd;
 	}
@@ -261,7 +265,7 @@ read_dasd_pt(int fd, struct slice all, struct slice *sp, int ns)
 				fmt_size = sectors512(vlabel.formatted_blocks,
 						      blocksize);
 			} else if (!strcmp(info.type, "ECKD")) {
-				/* formated w/o large volume support */
+				/* formatted w/o large volume support */
 				fmt_size = geo.cylinders * geo.heads
 					* geo.sectors * (blocksize >> 9);
 			} else {
