@@ -2,7 +2,7 @@
 #define _BLACKLIST_H
 
 #include <libudev.h>
-#include "regex.h"
+#include <regex.h>
 
 #define MATCH_NOTHING        0
 #define MATCH_WWID_BLIST     1
@@ -32,7 +32,7 @@ struct blentry_device {
 int setup_default_blist (struct config *);
 int alloc_ble_device (vector);
 int filter_devnode (vector, vector, char *);
-int filter_wwid (vector, vector, char *);
+int filter_wwid (vector, vector, char *, char *);
 int filter_device (vector, vector, char *, char *);
 int filter_path (struct config *, struct path *);
 int filter_property(struct config *, struct udev_device *);
