@@ -43,6 +43,7 @@ struct checker * alloc_checker (void)
 	if (c) {
 		INIT_LIST_HEAD(&c->node);
 		c->refcount = 1;
+		c->fd = -1;
 	}
 	return c;
 }
@@ -102,6 +103,8 @@ struct checker * add_checker (char *multipath_dir, char * name)
 	if (!c)
 		return NULL;
 	snprintf(c->name, CHECKER_NAME_LEN, "%s", name);
+	if (!strncmp(c->name, NONE, 4))
+		goto done;
 	snprintf(libname, LIB_CHECKER_NAMELEN, "%s/libcheck%s.so",
 		 multipath_dir, name);
 	if (stat(libname,&stbuf) < 0) {
@@ -145,8 +148,8 @@ struct checker * add_checker (char *multipath_dir, char * name)
 		condlog(0, "A dynamic linking error occurred: (%s)", errstr);
 	if (!c->repair)
 		goto out;
-
-	c->fd = 0;
+done:
+	c->fd = -1;
 	c->sync = 1;
 	list_add(&c->node, &checkers);
 	return c;
@@ -195,25 +198,33 @@ int checker_init (struct checker * c, void ** mpctxt_addr)
 	if (!c)
 		return 1;
 	c->mpcontext = mpctxt_addr;
-	return c->init(c);
+	if (c->init)
+		return c->init(c);
+	return 0;
+}
+
+void checker_clear (struct checker *c)
+{
+	memset(c, 0x0, sizeof(struct checker));
+	c->fd = -1;
 }
 
 void checker_put (struct checker * dst)
 {
 	struct checker * src;
 
-	if (!dst || !dst->check)
+	if (!dst || !strlen(dst->name))
 		return;
 	src = checker_lookup(dst->name);
 	if (dst->free)
 		dst->free(dst);
-	memset(dst, 0x0, sizeof(struct checker));
+	checker_clear(dst);
 	free_checker(src);
 }
 
 void checker_repair (struct checker * c)
 {
-	if (!c || !checker_selected(c))
+	if (!checker_selected(c))
 		return;
 
 	c->message[0] = '\0';
@@ -221,11 +232,11 @@ void checker_repair (struct checker * c)
 		MSG(c, "checker disabled");
 		return;
 	}
-
-	c->repair(c);
+	if (c->repair)
+		c->repair(c);
 }
 
-int checker_check (struct checker * c)
+int checker_check (struct checker * c, int path_state)
 {
 	int r;
 
@@ -237,7 +248,10 @@ int checker_check (struct checker * c)
 		MSG(c, "checker disabled");
 		return PATH_UNCHECKED;
 	}
-	if (c->fd <= 0) {
+	if (!strncmp(c->name, NONE, 4))
+		return path_state;
+
+	if (c->fd < 0) {
 		MSG(c, "no usable fd");
 		return PATH_WILD;
 	}
@@ -250,6 +264,8 @@ int checker_selected (struct checker * c)
 {
 	if (!c)
 		return 0;
+	if (!strncmp(c->name, NONE, 4))
+		return 1;
 	return (c->check) ? 1 : 0;
 }
 
