@@ -11,6 +11,7 @@
 #include "../third-party/valgrind/drd.h"
 #include "vector.h"
 #include "config.h"
+#include "defaults.h"
 
 void dlog (int sink, int prio, const char * fmt, ...)
 {
@@ -21,7 +22,7 @@ void dlog (int sink, int prio, const char * fmt, ...)
 	va_start(ap, fmt);
 	conf = get_multipath_config();
 	ANNOTATE_IGNORE_READS_BEGIN();
-	thres = (conf) ? conf->verbosity : 0;
+	thres = (conf) ? conf->verbosity : DEFAULT_VERBOSITY;
 	ANNOTATE_IGNORE_READS_END();
 	put_multipath_config(conf);
 
