@@ -26,6 +26,55 @@
  * Moreover, if a device needs a special treatment by the SCSI
  * subsystem it should be included in drivers/scsi/scsi_devinfo.c
  */
+
+#if 0
+	/*
+	 * Copy this TEMPLATE to add new hardware.
+	 *
+	 * Keep only mandatory(.vendor and .product) and modified attributes.
+	 * Attributes with default values must be removed.
+	 * .vendor, .product, .revision and .bl_product are POSIX Extended regex.
+	 *
+	 * COMPANY_NAME
+	 *
+	 * Maintainer : XXX
+	 * Mail : XXX
+	 */
+	{
+		/* If product-ID is different from marketing name add a comment */
+		.vendor        = "VENDOR",
+		.product       = "PRODUCT",
+		.revision      = "REVISION",
+		.bl_product    = "BL_PRODUCT",
+		.pgpolicy      = FAILOVER,
+		.uid_attribute = "ID_SERIAL",
+		.selector      = "service-time 0",
+		.checker_name  = TUR,
+		.alias_prefix  = "mpath",
+		.features      = "0",
+		.hwhandler     = "0",
+		.prio_name     = PRIO_CONST,
+		.prio_args     = "",
+		.pgfailback    = -FAILBACK_MANUAL,
+		.rr_weight     = RR_WEIGHT_NONE,
+		.no_path_retry = NO_PATH_RETRY_UNDEF,
+		.minio         = 1000,
+		.minio_rq      = 1,
+		.flush_on_last_del = FLUSH_DISABLED,
+		.user_friendly_names = USER_FRIENDLY_NAMES_OFF,
+		.fast_io_fail  = 5,
+		.dev_loss      = 600,
+		.retain_hwhandler = RETAIN_HWHANDLER_ON,
+		.detect_prio   = DETECT_PRIO_ON,
+		.detect_checker = DETECT_CHECKER_ON,
+		.deferred_remove = DEFERRED_REMOVE_OFF,
+		.delay_watch_checks = DELAY_CHECKS_OFF,
+		.delay_wait_checks = DELAY_CHECKS_OFF,
+		.skip_kpartx   = SKIP_KPARTX_OFF,
+		.max_sectors_kb = MAX_SECTORS_KB_UNDEF,
+	},
+#endif
+
 static struct hwentry default_hw[] = {
 	/*
 	 * Apple
@@ -49,6 +98,8 @@ static struct hwentry default_hw[] = {
 		.hwhandler     = "1 alua",
 		.prio_name     = PRIO_ALUA,
 		.no_path_retry = 18,
+		.fast_io_fail  = 10,
+		.dev_loss      = MAX_DEV_LOSS_TMO,
 	},
 	{
 		/* RA8000 / ESA12000 */
@@ -112,9 +163,9 @@ static struct hwentry default_hw[] = {
 		.prio_name     = PRIO_ALUA,
 	},
 	{
-		/* MSA 1040/2040 family */
+		/* MSA 1040, 2040 and 2050 families */
 		.vendor        = "HP",
-		.product       = "MSA [12]040 SA[NS]",
+		.product       = "MSA [12]0[45]0 SA[NS]",
 		.pgpolicy      = GROUP_BY_PRIO,
 		.pgfailback    = -FAILBACK_IMMEDIATE,
 		.no_path_retry = 18,
@@ -147,9 +198,9 @@ static struct hwentry default_hw[] = {
 		.prio_name     = PRIO_ALUA,
 	},
 	{
-		/* StoreVirtual 4000 family */
+		/* StoreVirtual 4000 and 3200 families */
 		.vendor        = "LEFTHAND",
-		.product       = "^(P4000|iSCSIDisk)",
+		.product       = "(P4000|iSCSIDisk|FCDISK)",
 		.pgpolicy      = GROUP_BY_PRIO,
 		.pgfailback    = -FAILBACK_IMMEDIATE,
 		.no_path_retry = 18,
@@ -224,34 +275,10 @@ static struct hwentry default_hw[] = {
 		.pgpolicy      = MULTIBUS,
 		.no_path_retry = NO_PATH_RETRY_QUEUE,
 	},
+	{
 		/* MD Series */
-	{
 		.vendor        = "DELL",
-		.product       = "MD3000",
-		.bl_product    = "Universal Xport",
-		.pgpolicy      = GROUP_BY_PRIO,
-		.checker_name  = RDAC,
-		.features      = "2 pg_init_retries 50",
-		.hwhandler     = "1 rdac",
-		.prio_name     = PRIO_RDAC,
-		.pgfailback    = -FAILBACK_IMMEDIATE,
-		.no_path_retry = 30,
-	},
-	{
-		.vendor        = "DELL",
-		.product       = "(MD32xx|MD36xx)",
-		.bl_product    = "Universal Xport",
-		.pgpolicy      = GROUP_BY_PRIO,
-		.checker_name  = RDAC,
-		.features      = "2 pg_init_retries 50",
-		.hwhandler     = "1 rdac",
-		.prio_name     = PRIO_RDAC,
-		.pgfailback    = -FAILBACK_IMMEDIATE,
-		.no_path_retry = 30,
-	},
-	{
-		.vendor        = "DELL",
-		.product       = "(MD34xx|MD38xx)",
+		.product       = "^MD3",
 		.bl_product    = "Universal Xport",
 		.pgpolicy      = GROUP_BY_PRIO,
 		.checker_name  = RDAC,
@@ -285,12 +312,18 @@ static struct hwentry default_hw[] = {
 	},
 	{
 		.vendor        = "FUJITSU",
-		.product       = "E[248]000",
+		.product       = "E[234]000",
 		.pgpolicy      = GROUP_BY_PRIO,
 		.pgfailback    = -FAILBACK_IMMEDIATE,
 		.no_path_retry = 10,
 		.prio_name     = PRIO_ALUA,
 	},
+	{
+		.vendor        = "FUJITSU",
+		.product       = "E[68]000",
+		.pgpolicy      = MULTIBUS,
+		.no_path_retry = 10,
+	},
 	/*
 	 * Hitachi
 	 *
@@ -535,16 +568,15 @@ static struct hwentry default_hw[] = {
 	},
 	{
 		/* XIV Storage System / FlashSystem A9000/A9000R */
-		.vendor        = "IBM",
-		.product       = "2810XIV",
+		.vendor        = "(XIV|IBM)",
+		.product       = "(NEXTRA|2810XIV)",
 		.no_path_retry = NO_PATH_RETRY_QUEUE,
 		.pgpolicy      = MULTIBUS,
 	},
 	{
-		/* FlashSystem 710/720/810/820/840/900 */
-		.vendor        = "IBM",
-		.product       = "FlashSystem",
-		.no_path_retry = NO_PATH_RETRY_FAIL,
+		/* TMS RamSan / FlashSystem 710/720/810/820/840/900 */
+		.vendor        = "(TMS|IBM)",
+		.product       = "(RamSan|FlashSystem)",
 		.pgpolicy      = MULTIBUS,
 	},
 	{
@@ -764,9 +796,9 @@ static struct hwentry default_hw[] = {
 		.no_path_retry = 30,
 	},
 	{
-		/* 3510 / 6020 and 6120 */
+		/* (Dot Hill) 3310, 3320, 3510 and 3511 */
 		.vendor        = "SUN",
-		.product       = "(StorEdge 3510|T4)",
+		.product       = "StorEdge 3",
 		.pgpolicy      = MULTIBUS,
 	},
 	{
@@ -852,7 +884,7 @@ static struct hwentry default_hw[] = {
 	 * Intel
 	 */
 	{
-		.vendor	       = "(Intel|INTEL)",
+		.vendor        = "(Intel|INTEL)",
 		.product       = "Multi-Flex",
 		.bl_product    = "VTrak V-LUN",
 		.hwhandler     = "1 alua",
@@ -865,7 +897,7 @@ static struct hwentry default_hw[] = {
 	 * Linux-IO Target
 	 */
 	{
-		.vendor	       = "(LIO-ORG|SUSE)",
+		.vendor        = "(LIO-ORG|SUSE)",
 		.product       = "RBD",
 		.hwhandler     = "1 alua",
 		.pgpolicy      = GROUP_BY_PRIO,
@@ -877,7 +909,7 @@ static struct hwentry default_hw[] = {
 	 * DataCore
 	 */
 	{
-		.vendor	       = "DataCore",
+		.vendor        = "DataCore",
 		.product       = "SANmelody",
 		.pgpolicy      = GROUP_BY_PRIO,
 		.pgfailback    = -FAILBACK_IMMEDIATE,
@@ -885,7 +917,8 @@ static struct hwentry default_hw[] = {
 		.prio_name     = PRIO_ALUA,
 	},
 	{
-		.vendor	       = "DataCore",
+		/* SANsymphony */
+		.vendor        = "DataCore",
 		.product       = "Virtual Disk",
 		.pgpolicy      = GROUP_BY_PRIO,
 		.pgfailback    = -FAILBACK_IMMEDIATE,
@@ -899,17 +932,16 @@ static struct hwentry default_hw[] = {
 		.vendor        = "PURE",
 		.product       = "FlashArray",
 		.pgpolicy      = MULTIBUS,
-		.fast_io_fail  = 10,
-		.dev_loss      = 60,
 	},
 	/*
 	 * Huawei
 	 */
 	{
 		/* OceanStor V3 */
-		.vendor        = "(HUAWEI|HUASY)",
+		.vendor        = "HUAWEI",
 		.product       = "XSG1",
-		.pgpolicy      = MULTIBUS,
+		.pgpolicy      = GROUP_BY_PRIO,
+		.prio_name     = PRIO_ALUA,
 	},
 	/*
 	 * Red Hat
@@ -920,7 +952,6 @@ static struct hwentry default_hw[] = {
 	{
 		.vendor        = "Ceph",
 		.product       = "RBD",
-		.no_path_retry = NO_PATH_RETRY_FAIL,
 		.checker_name  = RBD,
 		.deferred_remove = DEFERRED_REMOVE_ON,
 	},
@@ -934,13 +965,24 @@ static struct hwentry default_hw[] = {
 	},
 	/*
 	 * Infinidat
+	 *
+	 * Maintainer: Arnon Yaari
+	 * Mail: arnony@infinidat.com
 	 */
 	{
 		.vendor        = "NFINIDAT",
 		.product       = "InfiniBox",
 		.pgpolicy      = GROUP_BY_PRIO,
-		.pgfailback    = -FAILBACK_IMMEDIATE,
+		.pgfailback    = 30,
 		.prio_name     = PRIO_ALUA,
+		.selector      = "round-robin 0",
+		.rr_weight     = RR_WEIGHT_PRIO,
+		.no_path_retry = NO_PATH_RETRY_FAIL,
+		.minio         = 1,
+		.minio_rq      = 1,
+		.flush_on_last_del = FLUSH_ENABLED,
+		.fast_io_fail  = 15,
+		.dev_loss      = 15,
 	},
 	/*
 	 * Nimble Storage
@@ -995,6 +1037,16 @@ static struct hwentry default_hw[] = {
 		.prio_name     = PRIO_ALUA,
 		.no_path_retry = 15,
 	},
+	{
+		/* NST / UNITY */
+		.vendor        = "Nexsan",
+		.product       = "(NestOS|NST5000)",
+		.hwhandler     = "1 alua",
+		.pgpolicy      = GROUP_BY_PRIO,
+		.pgfailback    = -FAILBACK_IMMEDIATE,
+		.prio_name     = PRIO_ALUA,
+		.no_path_retry = 30,
+	},
 	/*
 	 * Xiotech
 	 */
@@ -1005,6 +1057,20 @@ static struct hwentry default_hw[] = {
 		.pgpolicy      = MULTIBUS,
 		.no_path_retry = 12,
 	},
+	{
+		/* iglu blaze family */
+		.vendor        = "(XIOTECH|XIOtech)",
+		.product       = "IGLU DISK",
+		.pgpolicy      = MULTIBUS,
+		.no_path_retry = 30,
+	},
+	{
+		/* Magnitude family */
+		.vendor        = "(XIOTECH|XIOtech)",
+		.product       = "Magnitude",
+		.pgpolicy      = MULTIBUS,
+		.no_path_retry = 30,
+	},
 	/*
 	 * Violin Memory
 	 */
@@ -1038,7 +1104,20 @@ static struct hwentry default_hw[] = {
 		.vendor        = "Promise",
 		.product       = "VTrak",
 		.bl_product    = "VTrak V-LUN",
-		.pgpolicy      = MULTIBUS,
+		.hwhandler     = "1 alua",
+		.pgpolicy      = GROUP_BY_PRIO,
+		.pgfailback    = -FAILBACK_IMMEDIATE,
+		.prio_name     = PRIO_ALUA,
+		.no_path_retry = 30,
+	},
+	{
+		.vendor        = "Promise",
+		.product       = "Vess",
+		.bl_product    = "Vess V-LUN",
+		.hwhandler     = "1 alua",
+		.pgpolicy      = GROUP_BY_PRIO,
+		.pgfailback    = -FAILBACK_IMMEDIATE,
+		.prio_name     = PRIO_ALUA,
 		.no_path_retry = 30,
 	},
 	/*
@@ -1053,50 +1132,54 @@ static struct hwentry default_hw[] = {
 		.prio_name     = PRIO_ALUA,
 		.no_path_retry = 30,
 	},
-#if 0
 	/*
-	 * Copy this TEMPLATE to add new hardware.
-	 *
-	 * Keep only mandatory(.vendor and .product) and modified attributes.
-	 * Attributes with default values must be removed.
-	 * .vendor, .product, .revision and .bl_product are POSIX Extended regex.
-	 *
-	 * COMPANY_NAME
-	 *
-	 * Maintainer : XXX
-	 * Mail : XXX
+	 * Generic NVMe devices
 	 */
 	{
-		/* If product-ID is different from marketing name add a comment */
-		.vendor        = "VENDOR",
-		.product       = "PRODUCT",
-		.revision      = "REVISION",
-		.bl_product    = "BL_PRODUCT",
-		.pgpolicy      = FAILOVER,
-		.uid_attribute = "ID_SERIAL",
-		.selector      = "service-time 0",
-		.checker_name  = TUR,
-		.alias_prefix  = "mpath",
-		.features      = "0",
-		.hwhandler     = "0",
-		.prio_name     = PRIO_CONST,
-		.prio_args     = "",
-		.pgfailback    = -FAILBACK_MANUAL,
-		.rr_weight     = RR_WEIGHT_NONE,
-		.no_path_retry = NO_PATH_RETRY_UNDEF,
-		.minio         = 1000,
-		.minio_rq      = 1,
-		.flush_on_last_del = FLUSH_DISABLED,
-		.user_friendly_names = USER_FRIENDLY_NAMES_OFF,
-		.fast_io_fail  = 5,
-		.dev_loss      = 600,
-		.retain_hwhandler = RETAIN_HWHANDLER_ON,
-		.detect_prio   = DETECT_PRIO_ON,
-		.deferred_remove = DEFERRED_REMOVE_OFF,
-		.delay_watch_checks = DELAY_CHECKS_OFF,
-		.delay_wait_checks = DELAY_CHECKS_OFF,
+		.vendor        = "NVME",
+		.product       = ".*",
+		.uid_attribute = "ID_WWN",
+		.checker_name  = NONE,
+		.retain_hwhandler = RETAIN_HWHANDLER_OFF,
+	},
+	/*
+	 * Dot Hill Systems - Seagate Technology
+	 */
+	{
+		/* SANnet family */
+		.vendor        = "DotHill",
+		.product       = "SANnet",
+		.pgpolicy      = MULTIBUS,
+		.no_path_retry = 30,
+	},
+	{
+		/* R/Evolution family */
+		.vendor        = "DotHill",
+		.product       = "R/Evo",
+		.pgpolicy      = GROUP_BY_PRIO,
+		.pgfailback    = -FAILBACK_IMMEDIATE,
+		.prio_name     = PRIO_ALUA,
+		.no_path_retry = 30,
+	},
+	{
+		/* AssuredSAN family */
+		.vendor        = "DotHill",
+		.product       = "^DH",
+		.pgpolicy      = GROUP_BY_PRIO,
+		.pgfailback    = -FAILBACK_IMMEDIATE,
+		.prio_name     = PRIO_ALUA,
+		.no_path_retry = 30,
+	},
+	/*
+	 * AccelStor
+	 */
+	{
+		/* NeoSapphire */
+		.vendor        = "AStor",
+		.product       = "NeoSapphire",
+		.pgpolicy      = MULTIBUS,
+		.no_path_retry = 30,
 	},
-#endif
 	/*
 	 * EOL
 	 */
@@ -1106,8 +1189,7 @@ static struct hwentry default_hw[] = {
 	},
 };
 
-extern int
-setup_default_hwtable (vector hw)
+int setup_default_hwtable(vector hw)
 {
 	int r = 0;
 	struct hwentry * hwe = default_hw;
