@@ -1,8 +1,8 @@
 #ifndef LOG_H
 #define LOG_H
 
-#define DEFAULT_AREA_SIZE 8192
-#define MAX_MSG_SIZE 128
+#define DEFAULT_AREA_SIZE 16384
+#define MAX_MSG_SIZE 256
 
 #ifndef LOGLEVEL
 #define LOGLEVEL 5
@@ -17,7 +17,7 @@
 struct logmsg {
 	short int prio;
 	void * next;
-	char * str;
+	char str[0];
 };
 
 struct logarea {
@@ -29,10 +29,11 @@ struct logarea {
 	char * buff;
 };
 
-struct logarea * la;
+extern struct logarea* la;
 
 int log_init (char * progname, int size);
 void log_close (void);
+void log_reset (char * progname);
 int log_enqueue (int prio, const char * fmt, va_list ap);
 int log_dequeue (void *);
 void log_syslog (void *);
