@@ -18,7 +18,9 @@
 #include "prio.h"
 #include "discovery.h"
 #include "dict.h"
+#include "util.h"
 #include "prioritizers/alua_rtpg.h"
+#include "prkey.h"
 #include <inttypes.h>
 
 pgpolicyfn *pgpolicies[] = {
@@ -41,28 +43,28 @@ do {									\
 #define do_default(dest, value)						\
 do {									\
 	dest = value;							\
-	origin = "(internal default)";					\
+	origin = "(setting: multipath internal)";			\
 } while(0)
 
 #define mp_set_mpe(var)							\
-do_set(var, mp->mpe, mp->var, "(LUN setting)")
+do_set(var, mp->mpe, mp->var, "(setting: multipath.conf multipaths section)")
 #define mp_set_hwe(var)							\
-do_set(var, mp->hwe, mp->var, "(controller setting)")
+do_set(var, mp->hwe, mp->var, "(setting: storage device configuration)")
 #define mp_set_ovr(var)							\
-do_set(var, conf->overrides, mp->var, "(overrides setting)")
+do_set(var, conf->overrides, mp->var, "(setting: multipath.conf overrides section)")
 #define mp_set_conf(var)						\
-do_set(var, conf, mp->var, "(config file default)")
+do_set(var, conf, mp->var, "(setting: multipath.conf defaults/devices section)")
 #define mp_set_default(var, value)					\
 do_default(mp->var, value)
 
 #define pp_set_mpe(var)							\
-do_set(var, mpe, pp->var, "(LUN setting)")
+do_set(var, mpe, pp->var, "(setting: multipath.conf multipaths section)")
 #define pp_set_hwe(var)							\
-do_set(var, pp->hwe, pp->var, "(controller setting)")
+do_set(var, pp->hwe, pp->var, "(setting: storage device configuration)")
 #define pp_set_conf(var)						\
-do_set(var, conf, pp->var, "(config file default)")
+do_set(var, conf, pp->var, "(setting: multipath.conf defaults/devices section)")
 #define pp_set_ovr(var)							\
-do_set(var, conf->overrides, pp->var, "(overrides setting)")
+do_set(var, conf->overrides, pp->var, "(setting: multipath.conf overrides section)")
 #define pp_set_default(var, value)					\
 do_default(pp->var, value)
 
@@ -77,12 +79,21 @@ do {									\
 } while(0)
 
 #define set_attr_mpe(var, shift)					\
-do_attr_set(var, mp->mpe, shift, "(LUN setting)")
+do_attr_set(var, mp->mpe, shift, "(setting: multipath.conf multipaths section)")
 #define set_attr_conf(var, shift)					\
-do_attr_set(var, conf, shift, "(config file default)")
+do_attr_set(var, conf, shift, "(setting: multipath.conf defaults/devices section)")
 
-extern int
-select_mode (struct config *conf, struct multipath *mp)
+#define do_prkey_set(src, msg)						\
+do {									\
+	if (src && src->prkey_source != PRKEY_SOURCE_NONE) {		\
+		mp->prkey_source = src->prkey_source;			\
+		mp->reservation_key = src->reservation_key;		\
+		origin = msg;						\
+		goto out;						\
+	}								\
+} while (0)
+
+int select_mode(struct config *conf, struct multipath *mp)
 {
 	char *origin;
 
@@ -95,8 +106,7 @@ out:
 	return 0;
 }
 
-extern int
-select_uid (struct config *conf, struct multipath *mp)
+int select_uid(struct config *conf, struct multipath *mp)
 {
 	char *origin;
 
@@ -109,8 +119,7 @@ out:
 	return 0;
 }
 
-extern int
-select_gid (struct config *conf, struct multipath *mp)
+int select_gid(struct config *conf, struct multipath *mp)
 {
 	char *origin;
 
@@ -128,8 +137,7 @@ out:
  * traverse the configuration layers from most specific to most generic
  * stop at first explicit setting found
  */
-extern int
-select_rr_weight (struct config *conf, struct multipath * mp)
+int select_rr_weight(struct config *conf, struct multipath * mp)
 {
 	char *origin, buff[13];
 
@@ -144,8 +152,7 @@ out:
 	return 0;
 }
 
-extern int
-select_pgfailback (struct config *conf, struct multipath * mp)
+int select_pgfailback(struct config *conf, struct multipath * mp)
 {
 	char *origin, buff[13];
 
@@ -160,14 +167,13 @@ out:
 	return 0;
 }
 
-extern int
-select_pgpolicy (struct config *conf, struct multipath * mp)
+int select_pgpolicy(struct config *conf, struct multipath * mp)
 {
 	char *origin, buff[POLICY_NAME_SIZE];
 
 	if (conf->pgpolicy_flag > 0) {
 		mp->pgpolicy = conf->pgpolicy_flag;
-		origin = "(cmd line flag)";
+		origin = "(setting: multipath command line [-p] flag)";
 		goto out;
 	}
 	mp_set_mpe(pgpolicy);
@@ -182,8 +188,7 @@ out:
 	return 0;
 }
 
-extern int
-select_selector (struct config *conf, struct multipath * mp)
+int select_selector(struct config *conf, struct multipath * mp)
 {
 	char *origin;
 
@@ -221,13 +226,13 @@ want_user_friendly_names(struct config *conf, struct multipath * mp)
 	int user_friendly_names;
 
 	do_set(user_friendly_names, mp->mpe, user_friendly_names,
-	       "(LUN setting)");
+	       "(setting: multipath.conf multipaths section)");
 	do_set(user_friendly_names, conf->overrides, user_friendly_names,
-	       "(overrides setting)");
+	       "(setting: multipath.conf overrides section)");
 	do_set(user_friendly_names, mp->hwe, user_friendly_names,
-	       "(controller setting)");
+	       "(setting: storage device configuration)");
 	do_set(user_friendly_names, conf, user_friendly_names,
-	       "(config file setting)");
+	       "(setting: multipath.conf defaults/devices section)");
 	do_default(user_friendly_names, DEFAULT_USER_FRIENDLY_NAMES);
 out:
 	condlog(3, "%s: user_friendly_names = %s %s", mp->wwid,
@@ -236,14 +241,13 @@ out:
 	return (user_friendly_names == USER_FRIENDLY_NAMES_ON);
 }
 
-extern int
-select_alias (struct config *conf, struct multipath * mp)
+int select_alias(struct config *conf, struct multipath * mp)
 {
 	char *origin = NULL;
 
 	if (mp->mpe && mp->mpe->alias) {
 		mp->alias = STRDUP(mp->mpe->alias);
-		origin = "(LUN setting)";
+		origin = "(setting: multipath.conf multipaths section)";
 		goto out;
 	}
 
@@ -258,26 +262,79 @@ select_alias (struct config *conf, struct multipath * mp)
 				mp->alias_old, mp->alias_prefix,
 				conf->bindings_read_only);
 		memset (mp->alias_old, 0, WWID_SIZE);
-		origin = "(using existing alias)";
+		origin = "(setting: using existing alias)";
 	}
 
 	if (mp->alias == NULL) {
 		mp->alias = get_user_friendly_alias(mp->wwid,
 				conf->bindings_file, mp->alias_prefix, conf->bindings_read_only);
-		origin = "(user_friendly_name)";
+		origin = "(setting: user_friendly_name)";
 	}
 out:
 	if (mp->alias == NULL) {
 		mp->alias = STRDUP(mp->wwid);
-		origin = "(default to wwid)";
+		origin = "(setting: default to WWID)";
 	}
 	if (mp->alias)
 		condlog(3, "%s: alias = %s %s", mp->wwid, mp->alias, origin);
 	return mp->alias ? 0 : 1;
 }
 
-extern int
-select_features (struct config *conf, struct multipath * mp)
+void reconcile_features_with_options(const char *id, char **features, int* no_path_retry,
+		  int *retain_hwhandler)
+{
+	static const char q_i_n_p[] = "queue_if_no_path";
+	static const char r_a_h_h[] = "retain_attached_hw_handler";
+	char buff[12];
+
+	if (*features == NULL)
+		return;
+	if (id == NULL)
+		id = "UNKNOWN";
+
+	/*
+	 * We only use no_path_retry internally. The "queue_if_no_path"
+	 * device-mapper feature is derived from it when the map is loaded.
+	 * For consistency, "queue_if_no_path" is removed from the
+	 * internal libmultipath features string.
+	 * For backward compatibility we allow 'features "1 queue_if_no_path"';
+	 * it's translated into "no_path_retry queue" here.
+	 */
+	if (strstr(*features, q_i_n_p)) {
+		condlog(0, "%s: option 'features \"1 %s\"' is deprecated, "
+			"please use 'no_path_retry queue' instead",
+			id, q_i_n_p);
+		if (*no_path_retry == NO_PATH_RETRY_UNDEF) {
+			*no_path_retry = NO_PATH_RETRY_QUEUE;
+			print_no_path_retry(buff, sizeof(buff),
+					    no_path_retry);
+			condlog(3, "%s: no_path_retry = %s (inherited setting from feature '%s')",
+				id, buff, q_i_n_p);
+		};
+		/* Warn only if features string is overridden */
+		if (*no_path_retry != NO_PATH_RETRY_QUEUE) {
+			print_no_path_retry(buff, sizeof(buff),
+					    no_path_retry);
+			condlog(2, "%s: ignoring feature '%s' because no_path_retry is set to '%s'",
+				id, q_i_n_p, buff);
+		}
+		remove_feature(features, q_i_n_p);
+	}
+	if (strstr(*features, r_a_h_h)) {
+		condlog(0, "%s: option 'features \"1 %s\"' is deprecated",
+			id, r_a_h_h);
+		if (*retain_hwhandler == RETAIN_HWHANDLER_UNDEF) {
+			condlog(3, "%s: %s = on (inherited setting from feature '%s')",
+				id, r_a_h_h, r_a_h_h);
+			*retain_hwhandler = RETAIN_HWHANDLER_ON;
+		} else if (*retain_hwhandler == RETAIN_HWHANDLER_OFF)
+			condlog(2, "%s: ignoring feature '%s' because %s is set to 'off'",
+				id, r_a_h_h, r_a_h_h);
+		remove_feature(features, r_a_h_h);
+	}
+}
+
+int select_features(struct config *conf, struct multipath *mp)
 {
 	char *origin;
 
@@ -288,22 +345,15 @@ select_features (struct config *conf, struct multipath * mp)
 	mp_set_default(features, DEFAULT_FEATURES);
 out:
 	mp->features = STRDUP(mp->features);
-	condlog(3, "%s: features = \"%s\" %s", mp->alias, mp->features, origin);
 
-	if (strstr(mp->features, "queue_if_no_path")) {
-		if (mp->no_path_retry == NO_PATH_RETRY_UNDEF)
-			mp->no_path_retry = NO_PATH_RETRY_QUEUE;
-		else if (mp->no_path_retry == NO_PATH_RETRY_FAIL) {
-			condlog(1, "%s: config error, overriding 'no_path_retry' value",
-				mp->alias);
-			mp->no_path_retry = NO_PATH_RETRY_QUEUE;
-		}
-	}
+	reconcile_features_with_options(mp->alias, &mp->features,
+					&mp->no_path_retry,
+					&mp->retain_hwhandler);
+	condlog(3, "%s: features = \"%s\" %s", mp->alias, mp->features, origin);
 	return 0;
 }
 
-extern int
-select_hwhandler (struct config *conf, struct multipath * mp)
+int select_hwhandler(struct config *conf, struct multipath *mp)
 {
 	char *origin;
 
@@ -317,40 +367,75 @@ out:
 	return 0;
 }
 
-extern int
-select_checker(struct config *conf, struct path *pp)
+/*
+ * Current RDAC (NetApp E-Series) firmware relies
+ * on periodic REPORT TARGET PORT GROUPS for
+ * internal load balancing.
+ * Using the sysfs priority checker defeats this purpose.
+ *
+ * Moreover, NetApp would also prefer the RDAC checker over ALUA.
+ * (https://www.redhat.com/archives/dm-devel/2017-September/msg00326.html)
+ */
+static int
+check_rdac(struct path * pp)
+{
+	int len;
+	char buff[44];
+
+	len = get_vpd_sgio(pp->fd, 0xC9, buff, 44);
+	if (len <= 0)
+		return 0;
+	return !(memcmp(buff + 4, "vac1", 4));
+}
+
+int select_checker(struct config *conf, struct path *pp)
 {
 	char *origin, *checker_name;
 	struct checker * c = &pp->checker;
 
-	do_set(checker_name, conf->overrides, checker_name, "(overrides setting)");
-	do_set(checker_name, pp->hwe, checker_name, "(controller setting)");
-	do_set(checker_name, conf, checker_name, "(config file setting)");
+	if (pp->detect_checker == DETECT_CHECKER_ON) {
+		origin = "(setting: storage device autodetected)";
+		if (check_rdac(pp)) {
+			checker_name = RDAC;
+			goto out;
+		} else if (pp->tpgs > 0) {
+			checker_name = TUR;
+			goto out;
+		}
+ 	}
+	do_set(checker_name, conf->overrides, checker_name, "(setting: multipath.conf overrides section)");
+	do_set(checker_name, pp->hwe, checker_name, "(setting: storage device configuration)");
+	do_set(checker_name, conf, checker_name, "(setting: multipath.conf defaults/devices section)");
 	do_default(checker_name, DEFAULT_CHECKER);
 out:
 	checker_get(conf->multipath_dir, c, checker_name);
 	condlog(3, "%s: path_checker = %s %s", pp->dev, c->name, origin);
 	if (conf->checker_timeout) {
 		c->timeout = conf->checker_timeout;
-		condlog(3, "%s: checker timeout = %u s (config file default)",
+		condlog(3, "%s: checker timeout = %u s (setting: multipath.conf defaults/devices section)",
 				pp->dev, c->timeout);
 	}
 	else if (sysfs_get_timeout(pp, &c->timeout) > 0)
-		condlog(3, "%s: checker timeout = %u ms (sysfs setting)",
+		condlog(3, "%s: checker timeout = %u s (setting: kernel sysfs)",
 				pp->dev, c->timeout);
 	else {
 		c->timeout = DEF_TIMEOUT;
-		condlog(3, "%s: checker timeout = %u ms (internal default)",
+		condlog(3, "%s: checker timeout = %u s (setting: multipath internal)",
 				pp->dev, c->timeout);
 	}
 	return 0;
 }
 
-extern int
-select_getuid (struct config *conf, struct path * pp)
+int select_getuid(struct config *conf, struct path *pp)
 {
 	char *origin;
 
+	pp->uid_attribute = parse_uid_attribute_by_attrs(conf->uid_attrs, pp->dev);
+	if (pp->uid_attribute) {
+		origin = "(setting: multipath.conf defaults section)";
+		goto out;
+	}
+
 	pp_set_ovr(getuid);
 	pp_set_ovr(uid_attribute);
 	pp_set_hwe(getuid);
@@ -371,23 +456,16 @@ out:
 void
 detect_prio(struct config *conf, struct path * pp)
 {
-	int ret;
 	struct prio *p = &pp->prio;
-	int tpgs = 0;
-	unsigned int timeout = conf->checker_timeout;
 	char buff[512];
 	char *default_prio = PRIO_ALUA;
 
-	if ((tpgs = get_target_port_group_support(pp->fd, timeout)) <= 0)
+	if (pp->tpgs <= 0)
 		return;
-	pp->tpgs = tpgs;
-	ret = get_target_port_group(pp, timeout);
-	if (ret < 0)
-		return;
-	if (get_asymmetric_access_state(pp->fd, ret, timeout) < 0)
-		return;
-	if (sysfs_get_asymmetric_access_state(pp, buff, 512) >= 0)
-		default_prio = PRIO_SYSFS;
+	if (pp->tpgs == 2 && !check_rdac(pp)) {
+		if (sysfs_get_asymmetric_access_state(pp, buff, 512) >= 0)
+			default_prio = PRIO_SYSFS;
+	}
 	prio_get(conf->multipath_dir, p, default_prio, DEFAULT_PRIO_ARGS);
 }
 
@@ -400,8 +478,7 @@ do {									\
 	}								\
 } while(0)
 
-extern int
-select_prio (struct config *conf, struct path * pp)
+int select_prio(struct config *conf, struct path *pp)
 {
 	char *origin;
 	struct mpentry * mpe;
@@ -410,17 +487,17 @@ select_prio (struct config *conf, struct path * pp)
 	if (pp->detect_prio == DETECT_PRIO_ON) {
 		detect_prio(conf, pp);
 		if (prio_selected(p)) {
-			origin = "(detected setting)";
+			origin = "(setting: storage device autodetected)";
 			goto out;
 		}
 	}
 	mpe = find_mpe(conf->mptable, pp->wwid);
-	set_prio(conf->multipath_dir, mpe, "(LUN setting)");
-	set_prio(conf->multipath_dir, conf->overrides, "(overrides setting)");
-	set_prio(conf->multipath_dir, pp->hwe, "controller setting)");
-	set_prio(conf->multipath_dir, conf, "(config file default)");
+	set_prio(conf->multipath_dir, mpe, "(setting: multipath.conf multipaths section)");
+	set_prio(conf->multipath_dir, conf->overrides, "(setting: multipath.conf overrides section)");
+	set_prio(conf->multipath_dir, pp->hwe, "(setting: storage device configuration)");
+	set_prio(conf->multipath_dir, conf, "(setting: multipath.conf defaults/devices section)");
 	prio_get(conf->multipath_dir, p, DEFAULT_PRIO, DEFAULT_PRIO_ARGS);
-	origin = "(internal default)";
+	origin = "(setting: multipath internal)";
 out:
 	/*
 	 * fetch tpgs mode for alua, if its not already obtained
@@ -438,14 +515,13 @@ out:
 	return 0;
 }
 
-extern int
-select_no_path_retry(struct config *conf, struct multipath *mp)
+int select_no_path_retry(struct config *conf, struct multipath *mp)
 {
 	char *origin = NULL;
 	char buff[12];
 
 	if (mp->flush_on_last_del == FLUSH_IN_PROGRESS) {
-		condlog(0, "flush_on_last_del in progress");
+		condlog(0, "%s: flush_on_last_del in progress", mp->alias);
 		mp->no_path_retry = NO_PATH_RETRY_FAIL;
 		return 0;
 	}
@@ -458,11 +534,8 @@ out:
 	if (origin)
 		condlog(3, "%s: no_path_retry = %s %s", mp->alias, buff,
 			origin);
-	else if (mp->no_path_retry != NO_PATH_RETRY_UNDEF)
-		condlog(3, "%s: no_path_retry = %s (inherited setting)",
-			mp->alias, buff);
 	else
-		condlog(3, "%s: no_path_retry = undef (internal default)",
+		condlog(3, "%s: no_path_retry = undef (setting: multipath internal)",
 			mp->alias);
 	return 0;
 }
@@ -472,10 +545,10 @@ select_minio_rq (struct config *conf, struct multipath * mp)
 {
 	char *origin;
 
-	do_set(minio_rq, mp->mpe, mp->minio, "(LUN setting)");
-	do_set(minio_rq, conf->overrides, mp->minio, "(overrides setting)");
-	do_set(minio_rq, mp->hwe, mp->minio, "(controller setting)");
-	do_set(minio_rq, conf, mp->minio, "(config file setting)");
+	do_set(minio_rq, mp->mpe, mp->minio, "(setting: multipath.conf multipaths section)");
+	do_set(minio_rq, conf->overrides, mp->minio, "(setting: multipath.conf overrides section)");
+	do_set(minio_rq, mp->hwe, mp->minio, "(setting: storage device configuration)");
+	do_set(minio_rq, conf, mp->minio, "(setting: multipath.conf defaults/devices section)");
 	do_default(mp->minio, DEFAULT_MINIO_RQ);
 out:
 	condlog(3, "%s: minio = %i %s", mp->alias, mp->minio, origin);
@@ -497,8 +570,7 @@ out:
 	return 0;
 }
 
-extern int
-select_minio (struct config *conf, struct multipath * mp)
+int select_minio(struct config *conf, struct multipath *mp)
 {
 	unsigned int minv_dmrq[3] = {1, 1, 0};
 
@@ -508,8 +580,7 @@ select_minio (struct config *conf, struct multipath * mp)
 		return select_minio_bio(conf, mp);
 }
 
-extern int
-select_fast_io_fail(struct config *conf, struct multipath *mp)
+int select_fast_io_fail(struct config *conf, struct multipath *mp)
 {
 	char *origin, buff[12];
 
@@ -523,8 +594,7 @@ out:
 	return 0;
 }
 
-extern int
-select_dev_loss(struct config *conf, struct multipath *mp)
+int select_dev_loss(struct config *conf, struct multipath *mp)
 {
 	char *origin, buff[12];
 
@@ -539,8 +609,7 @@ out:
 	return 0;
 }
 
-extern int
-select_flush_on_last_del(struct config *conf, struct multipath *mp)
+int select_flush_on_last_del(struct config *conf, struct multipath *mp)
 {
 	char *origin;
 
@@ -557,30 +626,45 @@ out:
 	return 0;
 }
 
-extern int
-select_reservation_key (struct config *conf, struct multipath * mp)
+int select_reservation_key(struct config *conf, struct multipath *mp)
 {
-	char *origin, buff[12];
+	char *origin, buff[PRKEY_SIZE];
+	char *from_file = "";
+	uint64_t prkey = 0;
 
-	mp_set_mpe(reservation_key);
-	mp_set_conf(reservation_key);
-	mp->reservation_key = NULL;
+	do_prkey_set(mp->mpe, "(setting: multipath.conf multipaths section)");
+	do_prkey_set(conf, "(setting: multipath.conf defaults/devices section)");
+	put_be64(mp->reservation_key, 0);
+	mp->prkey_source = PRKEY_SOURCE_NONE;
 	return 0;
 out:
-	print_reservation_key(buff, 12, &mp->reservation_key);
-	condlog(3, "%s: reservation_key = %s %s", mp->alias, buff, origin);
+	if (mp->prkey_source == PRKEY_SOURCE_FILE) {
+		from_file = " (from prkeys file)";
+		if (get_prkey(conf, mp, &prkey) != 0)
+			put_be64(mp->reservation_key, 0);
+		else
+			put_be64(mp->reservation_key, prkey);
+	}
+	print_reservation_key(buff, PRKEY_SIZE, mp->reservation_key,
+			      mp->prkey_source);
+	condlog(3, "%s: reservation_key = %s %s%s", mp->alias, buff, origin,
+		from_file);
 	return 0;
 }
 
-extern int
-select_retain_hwhandler (struct config *conf, struct multipath * mp)
+int select_retain_hwhandler(struct config *conf, struct multipath *mp)
 {
 	char *origin;
 	unsigned int minv_dm_retain[3] = {1, 5, 0};
 
 	if (!VERSION_GE(conf->version, minv_dm_retain)) {
 		mp->retain_hwhandler = RETAIN_HWHANDLER_OFF;
-		origin = "(requires kernel version >= 1.5.0)";
+		origin = "(setting: WARNING, requires kernel dm-mpath version >= 1.5.0)";
+		goto out;
+	}
+	if (get_linux_version_code() >= KERNEL_VERSION(4, 3, 0)) {
+		mp->retain_hwhandler = RETAIN_HWHANDLER_ON;
+		origin = "(setting: implied in kernel >= 4.3.0)";
 		goto out;
 	}
 	mp_set_ovr(retain_hwhandler);
@@ -594,8 +678,7 @@ out:
 	return 0;
 }
 
-extern int
-select_detect_prio (struct config *conf, struct path * pp)
+int select_detect_prio(struct config *conf, struct path *pp)
 {
 	char *origin;
 
@@ -609,14 +692,28 @@ out:
 	return 0;
 }
 
-extern int
-select_deferred_remove (struct config *conf, struct multipath *mp)
+int select_detect_checker(struct config *conf, struct path *pp)
+{
+	char *origin;
+
+	pp_set_ovr(detect_checker);
+	pp_set_hwe(detect_checker);
+	pp_set_conf(detect_checker);
+	pp_set_default(detect_checker, DEFAULT_DETECT_CHECKER);
+out:
+	condlog(3, "%s: detect_checker = %s %s", pp->dev,
+		(pp->detect_checker == DETECT_CHECKER_ON)? "yes" : "no",
+		origin);
+	return 0;
+}
+
+int select_deferred_remove(struct config *conf, struct multipath *mp)
 {
 	char *origin;
 
 #ifndef LIBDM_API_DEFERRED
 	mp->deferred_remove = DEFERRED_REMOVE_OFF;
-	origin = "(not compiled with support)";
+	origin = "(setting: WARNING, not compiled with support)";
 	goto out;
 #endif
 	if (mp->deferred_remove == DEFERRED_REMOVE_IN_PROGRESS) {
@@ -635,8 +732,7 @@ out:
 	return 0;
 }
 
-extern int
-select_delay_watch_checks(struct config *conf, struct multipath *mp)
+int select_delay_watch_checks(struct config *conf, struct multipath *mp)
 {
 	char *origin, buff[12];
 
@@ -646,13 +742,12 @@ select_delay_watch_checks(struct config *conf, struct multipath *mp)
 	mp_set_conf(delay_watch_checks);
 	mp_set_default(delay_watch_checks, DEFAULT_DELAY_CHECKS);
 out:
-	print_delay_checks(buff, 12, &mp->delay_watch_checks);
+	print_off_int_undef(buff, 12, &mp->delay_watch_checks);
 	condlog(3, "%s: delay_watch_checks = %s %s", mp->alias, buff, origin);
 	return 0;
 }
 
-extern int
-select_delay_wait_checks(struct config *conf, struct multipath *mp)
+int select_delay_wait_checks(struct config *conf, struct multipath *mp)
 {
 	char *origin, buff[12];
 
@@ -662,14 +757,77 @@ select_delay_wait_checks(struct config *conf, struct multipath *mp)
 	mp_set_conf(delay_wait_checks);
 	mp_set_default(delay_wait_checks, DEFAULT_DELAY_CHECKS);
 out:
-	print_delay_checks(buff, 12, &mp->delay_wait_checks);
+	print_off_int_undef(buff, 12, &mp->delay_wait_checks);
 	condlog(3, "%s: delay_wait_checks = %s %s", mp->alias, buff, origin);
 	return 0;
 
 }
 
-extern int
-select_skip_kpartx (struct config *conf, struct multipath * mp)
+int select_marginal_path_err_sample_time(struct config *conf, struct multipath *mp)
+{
+	char *origin, buff[12];
+
+	mp_set_mpe(marginal_path_err_sample_time);
+	mp_set_ovr(marginal_path_err_sample_time);
+	mp_set_hwe(marginal_path_err_sample_time);
+	mp_set_conf(marginal_path_err_sample_time);
+	mp_set_default(marginal_path_err_sample_time, DEFAULT_ERR_CHECKS);
+out:
+	print_off_int_undef(buff, 12, &mp->marginal_path_err_sample_time);
+	condlog(3, "%s: marginal_path_err_sample_time = %s %s", mp->alias, buff,
+			origin);
+	return 0;
+}
+
+int select_marginal_path_err_rate_threshold(struct config *conf, struct multipath *mp)
+{
+	char *origin, buff[12];
+
+	mp_set_mpe(marginal_path_err_rate_threshold);
+	mp_set_ovr(marginal_path_err_rate_threshold);
+	mp_set_hwe(marginal_path_err_rate_threshold);
+	mp_set_conf(marginal_path_err_rate_threshold);
+	mp_set_default(marginal_path_err_rate_threshold, DEFAULT_ERR_CHECKS);
+out:
+	print_off_int_undef(buff, 12, &mp->marginal_path_err_rate_threshold);
+	condlog(3, "%s: marginal_path_err_rate_threshold = %s %s", mp->alias, buff,
+			origin);
+	return 0;
+}
+
+int select_marginal_path_err_recheck_gap_time(struct config *conf, struct multipath *mp)
+{
+	char *origin, buff[12];
+
+	mp_set_mpe(marginal_path_err_recheck_gap_time);
+	mp_set_ovr(marginal_path_err_recheck_gap_time);
+	mp_set_hwe(marginal_path_err_recheck_gap_time);
+	mp_set_conf(marginal_path_err_recheck_gap_time);
+	mp_set_default(marginal_path_err_recheck_gap_time, DEFAULT_ERR_CHECKS);
+out:
+	print_off_int_undef(buff, 12, &mp->marginal_path_err_recheck_gap_time);
+	condlog(3, "%s: marginal_path_err_recheck_gap_time = %s %s", mp->alias, buff,
+			origin);
+	return 0;
+}
+
+int select_marginal_path_double_failed_time(struct config *conf, struct multipath *mp)
+{
+	char *origin, buff[12];
+
+	mp_set_mpe(marginal_path_double_failed_time);
+	mp_set_ovr(marginal_path_double_failed_time);
+	mp_set_hwe(marginal_path_double_failed_time);
+	mp_set_conf(marginal_path_double_failed_time);
+	mp_set_default(marginal_path_double_failed_time, DEFAULT_ERR_CHECKS);
+out:
+	print_off_int_undef(buff, 12, &mp->marginal_path_double_failed_time);
+	condlog(3, "%s: marginal_path_double_failed_time = %s %s", mp->alias, buff,
+			origin);
+	return 0;
+}
+
+int select_skip_kpartx (struct config *conf, struct multipath * mp)
 {
 	char *origin;
 
@@ -684,3 +842,24 @@ out:
 		origin);
 	return 0;
 }
+
+int select_max_sectors_kb(struct config *conf, struct multipath * mp)
+{
+	char *origin;
+
+	mp_set_mpe(max_sectors_kb);
+	mp_set_ovr(max_sectors_kb);
+	mp_set_hwe(max_sectors_kb);
+	mp_set_conf(max_sectors_kb);
+	mp_set_default(max_sectors_kb, DEFAULT_MAX_SECTORS_KB);
+	/*
+	 * In the default case, we will not modify max_sectors_kb in sysfs
+	 * (see sysfs_set_max_sectors_kb()).
+	 * Don't print a log message here to avoid user confusion.
+	 */
+	return 0;
+out:
+	condlog(3, "%s: max_sectors_kb = %i %s", mp->alias, mp->max_sectors_kb,
+		origin);
+	return 0;
+}
