@@ -71,7 +71,7 @@ write_out_wwid(int fd, char *wwid) {
 			strerror(errno));
 		return -1;
 	}
-	if (write_all(fd, buf, strlen(buf)) != strlen(buf)) {
+	if (write(fd, buf, strlen(buf)) != strlen(buf)) {
 		condlog(0, "cannot write wwid to wwids file : %s",
 			strerror(errno));
 		if (ftruncate(fd, offset))
@@ -89,8 +89,11 @@ replace_wwids(vector mp)
 	struct multipath * mpp;
 	size_t len;
 	int ret = -1;
+	struct config *conf;
 
+	conf = get_multipath_config();
 	fd = open_file(conf->wwids_file, &can_write, WWIDS_FILE_HEADER);
+	put_multipath_config(conf);
 	if (fd < 0)
 		goto out;
 	if (!can_write) {
@@ -107,7 +110,7 @@ replace_wwids(vector mp)
 		goto out_file;
 	}
 	len = strlen(WWIDS_FILE_HEADER);
-	if (write_all(fd, WWIDS_FILE_HEADER, len) != len) {
+	if (write(fd, WWIDS_FILE_HEADER, len) != len) {
 		condlog(0, "Can't write wwid file header : %s",
 			strerror(errno));
 		/* cleanup partially written header */
@@ -188,6 +191,7 @@ remove_wwid(char *wwid) {
 	int fd, len, can_write;
 	char *str;
 	int ret = -1;
+	struct config *conf;
 
 	len = strlen(wwid) + 4; /* two slashes the newline and a zero byte */
 	str = malloc(len);
@@ -201,7 +205,9 @@ remove_wwid(char *wwid) {
 		goto out;
 	}
 	condlog(3, "removing line '%s' from wwids file", str);
+	conf = get_multipath_config();
 	fd = open_file(conf->wwids_file, &can_write, WWIDS_FILE_HEADER);
+	put_multipath_config(conf);
 	if (fd < 0)
 		goto out;
 	if (!can_write) {
@@ -222,7 +228,11 @@ check_wwids_file(char *wwid, int write_wwid)
 {
 	int fd, can_write, found, ret;
 	FILE *f;
+	struct config *conf;
+
+	conf = get_multipath_config();
 	fd = open_file(conf->wwids_file, &can_write, WWIDS_FILE_HEADER);
+	put_multipath_config(conf);
 	if (fd < 0)
 		return -1;
 
@@ -260,6 +270,43 @@ out:
 	return ret;
 }
 
+int
+should_multipath(struct path *pp1, vector pathvec)
+{
+	int i, ignore_new_devs;
+	struct path *pp2;
+	struct config *conf;
+
+	conf = get_multipath_config();
+	ignore_new_devs = conf->ignore_new_devs;
+	if (!conf->find_multipaths && !ignore_new_devs) {
+		put_multipath_config(conf);
+		return 1;
+	}
+	put_multipath_config(conf);
+
+	condlog(4, "checking if %s should be multipathed", pp1->dev);
+	if (!ignore_new_devs) {
+		vector_foreach_slot(pathvec, pp2, i) {
+			if (pp1->dev == pp2->dev)
+				continue;
+			if (strncmp(pp1->wwid, pp2->wwid, WWID_SIZE) == 0) {
+				condlog(3, "found multiple paths with wwid %s, "
+					"multipathing %s", pp1->wwid, pp1->dev);
+				return 1;
+			}
+		}
+	}
+	if (check_wwids_file(pp1->wwid, 0) < 0) {
+		condlog(3, "wwid %s not in wwids file, skipping %s",
+			pp1->wwid, pp1->dev);
+		return 0;
+	}
+	condlog(3, "found wwid %s in wwids file, multipathing %s", pp1->wwid,
+		pp1->dev);
+	return 1;
+}
+
 int
 remember_wwid(char *wwid)
 {
